// Sound class for Sudoku...
//
// There are MANY ways to implement sound in a FLTK application.
// The approach we are using here is to conditionally compile OS-
// specific code into the application - CoreAudio for MacOS X, the
// standard Win32 API stuff for Windows, ALSA or X11 for Linux, and
// X11 for all others.  We have to support ALSA on Linux because the
// current Xorg releases no longer support XBell() or the PC speaker.
//
// There are several good cross-platform audio libraries we could also
// use, such as OpenAL, PortAudio, and SDL, however they were not chosen
// for this application because of our limited use of sound.
//
// Many thanks to Ian MacArthur who provided sample code that led to
// the CoreAudio implementation you see here!
#ifndef SUDOKUSOUND_H_
#define SUDOKUSOUND_H_

//solve mmsystem.h DWORD etc problems
#include <windows.h>

// Audio headers...
#include <config.h>

#ifndef WIN32
#  include <unistd.h>
#endif // !WIN32

#ifdef HAVE_ALSA_ASOUNDLIB_H
#  define ALSA_PCM_NEW_HW_PARAMS_API
#  include <alsa/asoundlib.h>
#endif // HAVE_ALSA_ASOUNDLIB_H
#ifdef __APPLE__
#  include <CoreAudio/AudioHardware.h>
#endif // __APPLE__
#ifdef WIN32
#  include <mmsystem.h>
#endif // WIN32

class SudokuSound
{	
  // Private, OS-specific data...
#ifdef __APPLE__
  AudioDeviceID device;
  AudioStreamBasicDescription format;
  short *data;
  int remaining;

  static OSStatus audio_cb(AudioDeviceID device,
						   const AudioTimeStamp *current_time,
						   const AudioBufferList *data_in,
						   const AudioTimeStamp *time_in,
						   AudioBufferList *data_out,
						   const AudioTimeStamp *time_out,
						   void *client_data);
#elif defined(WIN32)
  HWAVEOUT	device;
  HGLOBAL	header_handle;
  LPWAVEHDR	header_ptr;
  HGLOBAL	data_handle;
  LPSTR		data_ptr;

#else
#  ifdef HAVE_ALSA_ASOUNDLIB_H
  snd_pcm_t *handle;
#  endif // HAVE_ALSA_ASOUNDLIB_H
#endif // __APPLE__

  // Common data...
  static int frequencies[9];
  static short *sample_data[9];
  static int sample_size;
  int soundon_; //always on (inited in constructor) until switched off 
  
public:
  SudokuSound();
  virtual ~SudokuSound();
	
  void	play(char note);
  void	soundon(int on); //1 is on ;
};
#endif /*SUDOKUSOUND_H_*/
